#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  2021/11/13 17:50
@Desc    :  
"""

gitignore_content = """.idea/
.pytest_cache/
.tep_allure_tmp/
__pycache__/
*.pyc
reports/
debug/"""

conf_yaml_content = """env: qa"""

conftest_content = """#!/usr/bin/python
# encoding=utf-8

\"\"\" 只能管理员编辑，只对外提供fixture。
\"\"\"

import os

import pytest

# 项目目录路径
_project_dir = os.path.dirname(os.path.abspath(__file__))


# 设置缓存供tep使用
@pytest.fixture(scope="session", autouse=True)
def _project_cache(request):
    request.config.cache.set("project_dir", _project_dir)


# 自动导入fixtures
_fixtures_dir = os.path.join(_project_dir, "fixtures")
for root, _, files in os.walk(_fixtures_dir):
    for file in files:
        if file.startswith("fixture_") and file.endswith(".py"):
            full_path = os.path.join(root, file)
            import_path = full_path.replace(_fixtures_dir, "").replace("\\\\", ".").replace("/", ".").replace(".py", "")
            try:
                fixture_path = "fixtures" + import_path
                exec(f"from {fixture_path} import *")
            except:
                fixture_path = ".fixtures" + import_path
                exec(f"from {fixture_path} import *")
"""

pytest_ini_content = """[pytest]
markers =
    smoke: smoke test
    regress: regress test
"""

fixture_admin_content = """#!/usr/bin/python
# encoding=utf-8

\"\"\" Can only be modified by the administrator. Only fixtures are provided.
\"\"\"

from tep.fixture import *


@pytest.fixture
def common_created_by_admin():
    pass
"""

fixture_env_vars_content = """#!/usr/bin/python
# encoding=utf-8

from tep.dao import mysql_engine
from tep.fixture import *


@pytest.fixture(scope="session")
def env_vars(config):
    class Clazz(TepVars):
        env = config["env"]

        \"\"\"变量定义开始\"\"\"
        # 环境变量
        mapping = {
            \"qa\": {
                "domain": "http://127.0.0.1:5000",
                "mysql_engine": mysql_engine("127.0.0.1",  # host
                                             "2306",  # port
                                             "root",  # username
                                             "123456",  # password
                                             "qa"),  # db_name
            },
            "release": {
                "domain": "https://release.com",
                "mysql_engine": mysql_engine("127.0.0.1",
                                             "2306",
                                             "root",
                                             "123456",
                                             "release"),
            }
            # 继续添加
        }
        # 定义类属性，敲代码时会有智能提示
        domain = mapping[env]["domain"]
        mysql_engine = mapping[env]["mysql_engine"]
        \"\"\"变量定义结束\"\"\"

    return Clazz()
"""

fixture_login_content = """from tep.client import request
from tep.fixture import *


def _jwt_headers(token):
    return {"Content-Type": "application/json", "authorization": f"Bearer {token}"}


@pytest.fixture(scope="session")
def login(env_vars):
    # Code your login
    logger.info("Administrator login")
    response = request(
        "post",
        url=env_vars.domain + "/login",
        headers={"Content-Type": "application/json"},
        json={
            "username": "dongfanger",
            "password": "123456",
        }
    )
    assert response.status_code < 400
    response_token = jmespath.search("token", response.json())

    class Clazz:
        token = response_token
        jwt_headers = _jwt_headers(response_token)

    return Clazz
"""

fixture_your_name_content = """#!/usr/bin/python
# encoding=utf-8

\"\"\" Please define your own fixture.
\"\"\"

from tep.fixture import *


@pytest.fixture(scope="session")
def env_vars_your_name(config):
    class Clazz:
        env = config["env"]

        # Environment and variables
        mapping = {
            "qa": {
                "your_var": "123",
            },
            "release": {
                "your_var": "456",
            }
            # Add your environment and variables
        }
        # Define properties for auto display
        your_var = mapping[env]["your_var"]

    return Clazz()


@pytest.fixture
def share_your_name():
    pass
"""

fixture_second_content = """#!/usr/bin/python
# encoding=utf-8


from tep.fixture import *


@pytest.fixture
def fixture_second():
    logger.info("fixture_second")
"""

fixture_three_content = """#!/usr/bin/python
# encoding=utf-8


from tep.fixture import *


@pytest.fixture
def fixture_three():
    logger.info("fixture_three")
"""

test_login_content = """from loguru import logger


def test_login(login):
    logger.info(login.token)
"""

test_post_content = """import jmespath
from loguru import logger

from tep.client import request


def test_post(faker_ch, url, login):
    # description
    logger.info("test post")
    # data
    fake = faker_ch
    # request
    response = request(
        "post",
        url=url("/api/users"),
        headers=login.jwt_headers,
        json={
            "name": fake.name()
        }
    )
    # assert
    assert response.status_code < 400
    # extract
    user_id = jmespath.search("id", response.json())
"""

test_mysql_content = """from loguru import logger
from tep.dao import print_db_table


def test_mysql(pd, env_vars):
    data = pd.read_sql("select 1 from dual", env_vars.mysql_engine)
    logger.info(print_db_table(data))
"""

test_request_content = """from tep.client import request

request("get", url="", headers={}, json={})
request("post", url="", headers={}, params={})
request("put", url="", headers={}, json={})
request("delete", url="", headers={})

# upload excel
file_name = ""
file_path = ""
request("post",
        url="",
        headers={},
        files={
            "file": (
                file_name,
                open(file_path, "rb"),
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
            )
        },
        verify=False
        )
"""

fastapi_mock_content = """#!/usr/bin/python
# encoding=utf-8

import uvicorn
from fastapi import FastAPI, Request

app = FastAPI()


@app.post("/login")
async def login(req: Request):
    body = await req.json()
    if body["username"] == "dongfanger" and body["password"] == "123456":
        return {"token": "de2e3ffu29"}
    return ""


@app.get("/searchSku")
def search_sku(req: Request):
    if req.headers.get("token") == "de2e3ffu29" and req.query_params.get("skuName") == "电子书":
        return {"skuId": "222", "price": "2.3"}
    return ""


@app.post("/addCart")
async def add_cart(req: Request):
    body = await req.json()
    if req.headers.get("token") == "de2e3ffu29" and body["skuId"] == "222":
        return {"skuId": "222", "price": "2.3", "skuNum": "3", "totalPrice": "6.9"}
    return ""


@app.post("/order")
async def order(req: Request):
    body = await req.json()
    if req.headers.get("token") == "de2e3ffu29" and body["skuId"] == "222":
        return {"orderId": "333"}
    return ""


@app.post("/pay")
async def pay(req: Request):
    body = await req.json()
    if req.headers.get("token") == "de2e3ffu29" and body["orderId"] == "333":
        return {"success": "true"}
    return ""


if __name__ == '__main__':
    uvicorn.run("fastapi_mock:app", host="127.0.0.1", port=5000)
"""

http_client_content = """#!/usr/bin/python
# encoding=utf-8

import decimal
import json
import time

import allure
from loguru import logger
from tep import client
from tep.client import TepResponse


def request_monkey_patch(req, *args, **kwargs):
    start = time.process_time()
    desc = ""
    if "desc" in kwargs:
        desc = kwargs.get("desc")
        kwargs.pop("desc")
    response = req(*args, **kwargs)
    end = time.process_time()
    elapsed = str(decimal.Decimal("%.3f" % float(end - start))) + "s"
    log4a = "{}\\n{}status:{}\\nresponse:{}\\nelapsed:{}"
    try:
        kv = ""
        for k, v in kwargs.items():
            # if not json, str()
            try:
                v = json.dumps(v, ensure_ascii=False)
            except TypeError:
                v = str(v)
            kv += f"{k}:{v}\\n"
        args = list(args)
        args += ["", ""]
        method, url, *t = args
        method_url = ""
        if method:
            method_url = f'\\nmethod:"{method}" '
        if url:
            method_url += f'\\nurl:"{url}" '
        request_response = log4a.format(method_url, kv, response.status_code, response.text, elapsed)
        logger.info(request_response)
        allure.attach(request_response, f'{desc} request & response', allure.attachment_type.TEXT)
    except AttributeError:
        logger.error("request failed")
    except TypeError:
        logger.warning(log4a)
    return TepResponse(response)


def request(method, url, **kwargs):
    client.tep_request_monkey_patch = request_monkey_patch
    return client.request(method, url, **kwargs)
"""

test_login_pay_content = """import jmespath
from tep.client import request

\"\"\"
测试登录到下单流程，需要先运行utils/flask_mock_api.py
\"\"\"


def test(env_vars, login):
    # 搜索商品
    response = request(
        "get",
        url=env_vars.domain + "/searchSku",
        headers={"token": login.token},
        params={"skuName": "电子书"}
    )
    sku_id = jmespath.search("skuId", response.json())
    sku_price = jmespath.search("price", response.json())
    assert response.status_code < 400

    # 添加购物车
    sku_num = 3
    response = request(
        "post",
        url=env_vars.domain + "/addCart",
        headers={"token": login.token},
        json={"skuId": sku_id, "skuNum": str(sku_num)}
    )
    total_price = jmespath.search("totalPrice", response.json())
    assert response.status_code < 400

    # 下单
    response = request(
        "post",
        url=env_vars.domain + "/order",
        headers={"token": login.token},
        json={"skuId": sku_id, "price": sku_price, "skuNum": str(sku_num), "totalPrice": total_price}
    )
    order_id = jmespath.search("orderId", response.json())
    assert response.status_code < 400

    # 支付
    response = request(
        "post",
        url=env_vars.domain + "/pay",
        headers={"token": login.token},
        json={"orderId": order_id, "payAmount": "6.9"}
    )
    assert response.status_code < 400
    assert response.json()["success"] == "true"
"""

test_login_pay_httprunner_content = """from httprunner import HttpRunner, Config, Step, RunRequest

\"\"\"
测试登录到下单流程，需要先运行utils/flask_mock_api.py
\"\"\"


class TestLoginPay(HttpRunner):
    config = (
        Config("登录到下单流程")
            .variables(
            **{
                "skuNum": "3"
            }
        )
            .base_url("http://127.0.0.1:5000")
    )

    teststeps = [
        Step(
            RunRequest("登录")
                .post("/login")
                .with_headers(**{"Content-Type": "application/json"})
                .with_json({"username": "dongfanger", "password": "123456"})
                .extract()
                .with_jmespath("body.token", "token")
                .validate()
                .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("搜索商品")
                .get("searchSku?skuName=电子书")
                .with_headers(**{"token": "$token"})
                .extract()
                .with_jmespath("body.skuId", "skuId")
                .with_jmespath("body.price", "skuPrice")
                .validate()
                .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("添加购物车")
                .post("/addCart")
                .with_headers(**{"Content-Type": "application/json",
                                 "token": "$token"})
                .with_json({"skuId": "$skuId", "skuNum": "$skuNum"})
                .extract()
                .with_jmespath("body.totalPrice", "totalPrice")
                .validate()
                .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("下单")
                .post("/order")
                .with_headers(**{"Content-Type": "application/json",
                                 "token": "$token"})
                .with_json({"skuId": "$skuId", "price": "$skuPrice", "skuNum": "$skuNum", "totalPrice": "$totalPrice"})
                .extract()
                .with_jmespath("body.orderId", "orderId")
                .validate()
                .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("支付")
                .post("/pay")
                .with_headers(**{"Content-Type": "application/json",
                                 "token": "$token"})
                .with_json({"orderId": "$orderId", "payAmount": "6.9"})
                .validate()
                .assert_equal("status_code", 200)
                .assert_equal("body.success", "true")
        ),
    ]
"""

Login_content = """from tep.client import BaseRequest


class Login(BaseRequest):

    def post(self):
        response = self.request(
            "post",
            url=self.case_vars.get("domain") + "/login",
            headers={"Content-Type": "application/json"},
            json={
                "username": "dongfanger",
                "password": "123456",
            }
        )
        assert response.status_code < 400
        self.case_vars.put("token", response.jmespath("token"))
"""

SearchSku_content = """from tep.client import BaseRequest


class SearchSku(BaseRequest):

    def get(self):
        response = self.request(
            "get",
            url=self.case_vars.get("domain") + "/searchSku",
            headers={"token": self.case_vars.get("token")},
            params={"skuName": "电子书"}
        )
        self.case_vars.put("skuId", response.jmespath("skuId"))
        self.case_vars.put("skuPrice", response.jmespath("price"))
        assert response.status_code < 400
"""

AddCart_content = """from tep.client import BaseRequest


class AddCart(BaseRequest):

    def post(self):
        response = self.request(
            "post",
            url=self.case_vars.get("domain") + "/addCart",
            headers={"token": self.case_vars.get("token")},
            json={"skuId": self.case_vars.get("skuId"), "skuNum": self.case_vars.get("skuNum")}
        )
        self.case_vars.put("totalPrice", response.jmespath("totalPrice"))
        assert response.status_code < 400
"""

Order_content = """from tep.client import BaseRequest


class Order(BaseRequest):

    def post(self):
        response = self.request(
            "post",
            url=self.case_vars.get("domain") + "/order",
            headers={"token": self.case_vars.get("token")},
            json={"skuId": self.case_vars.get("skuId"), "price": self.case_vars.get("skuPrice"),
                  "skuNum": self.case_vars.get("skuNum"), "totalPrice": self.case_vars.get("totalPrice")}
        )
        self.case_vars.put("orderId", response.jmespath("orderId"))
        assert response.status_code < 400
"""

Pay_content = """from tep.client import BaseRequest


class Pay(BaseRequest):

    def post(self):
        response = self.request(
            "post",
            url=self.case_vars.get("domain") + "/pay",
            headers={"token": self.case_vars.get("token")},
            json={"orderId": self.case_vars.get("orderId"), "payAmount": "6.9"}
        )
        assert response.status_code < 400
        assert response.jmespath("success") == "true"
"""

test_login_pay_mvc_content = """from tep.fixture import TepVars

from services.AddCart import AddCart
from services.Login import Login
from services.Order import Order
from services.Pay import Pay
from services.SearchSku import SearchSku

\"\"\"
测试登录到下单流程，需要先运行utils / flask_mock_api.py
\"\"\"


class Test:
    case_vars = TepVars()
    case_vars.vars_ = {
        "domain": "http://127.0.0.1:5000",
        "skuNum": "3"
    }

    def test(self):
        # 登录
        Login(Test).post()
        # 搜索商品
        SearchSku(Test).get()
        # 添加购物车
        AddCart(Test).post()
        # 下单
        Order(Test).post()
        # 支付
        Pay(Test).post()
"""

test_multi_fixture_content = """#!/usr/bin/python
# encoding=utf-8


def test(fixture_second, fixture_three):
    pass
"""

test_request_monkey_patch_content = """from utils.http_client import request


def test_login(env_vars):
    response = request(
        "post",
        url=env_vars.domain + "/login",
        desc="登录",
        headers={"Content-Type": "application/json"},
        json={
            "username": "dongfanger",
            "password": "123456",
        }
    )
    assert response.status_code < 400
"""



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  7/23/2020 8:12 PM
@Desc    :
"""

import os
import platform
import sys

from loguru import logger

from tep.sample import *


class ExtraArgument:
    create_venv = False


def init_parser_scaffold(subparsers):
    sub_parser_scaffold = subparsers.add_parser(
        "startproject", help="Create a new project with template structure."
    )
    sub_parser_scaffold.add_argument(
        "project_name", type=str, nargs="?", help="Specify new project name."
    )
    sub_parser_scaffold.add_argument(
        "-venv",
        dest="create_venv",
        action="store_true",
        help="Create virtual environment in the project, and install tep.",
    )
    return sub_parser_scaffold


def create_scaffold(project_name):
    """ Create scaffold with specified project name.
    """
    if os.path.isdir(project_name):
        logger.warning(
            f"Project folder {project_name} exists, please specify a new project name."
        )
        return 1
    elif os.path.isfile(project_name):
        logger.warning(
            f"Project name {project_name} conflicts with existed file, please specify a new one."
        )
        return 1

    logger.info(f"Create new project: {project_name}")
    print(f"Project root dir: {os.path.join(os.getcwd(), project_name)}\n")

    def create_folder(path):
        os.makedirs(path)
        msg = f"Created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"Created file: {path}"
        print(msg)

    create_folder(project_name)
    create_folder(os.path.join(project_name, "fixtures"))
    create_folder(os.path.join(project_name, "fixtures", "second"))
    create_folder(os.path.join(project_name, "fixtures", "second", "three"))
    create_folder(os.path.join(project_name, "tests"))
    create_folder(os.path.join(project_name, "files"))
    create_folder(os.path.join(project_name, "reports"))
    create_folder(os.path.join(project_name, "utils"))
    create_folder(os.path.join(project_name, "services"))

    create_file(os.path.join(project_name, ".gitignore"), gitignore_content)
    create_file(os.path.join(project_name, "conf.yaml"), conf_yaml_content)
    create_file(os.path.join(project_name, "conftest.py"), conftest_content)
    create_file(os.path.join(project_name, "pytest.ini"), pytest_ini_content)
    create_file(os.path.join(project_name, "fixtures", "__init__.py"))
    create_file(os.path.join(project_name, "fixtures", "fixture_admin.py"), fixture_admin_content)
    create_file(os.path.join(project_name, "fixtures", "fixture_env_vars.py"), fixture_env_vars_content)
    create_file(os.path.join(project_name, "fixtures", "fixture_login.py"), fixture_login_content)
    create_file(os.path.join(project_name, "fixtures", "fixture_your_name.py"), fixture_your_name_content)
    create_file(os.path.join(project_name, "fixtures", "second", "__init__.py"))
    create_file(os.path.join(project_name, "fixtures", "second", "fixture_second.py"), fixture_second_content)
    create_file(os.path.join(project_name, "fixtures", "second", "three", "__init__.py"))
    create_file(os.path.join(project_name, "fixtures", "second", "three", "fixture_three.py"), fixture_three_content)
    create_file(os.path.join(project_name, "tests", "__init__.py"))
    create_file(os.path.join(project_name, "tests", "test_login.py"), test_login_content)
    create_file(os.path.join(project_name, "tests", "test_post.py"), test_post_content)
    create_file(os.path.join(project_name, "tests", "test_mysql.py"), test_mysql_content)
    create_file(os.path.join(project_name, "tests", "test_request.py"), test_request_content)
    create_file(os.path.join(project_name, "tests", "test_login_pay.py"), test_login_pay_content)
    create_file(os.path.join(project_name, "tests", "test_login_pay_httprunner.py"), test_login_pay_httprunner_content)
    create_file(os.path.join(project_name, "tests", "test_login_pay_mvc.py"), test_login_pay_mvc_content)
    create_file(os.path.join(project_name, "tests", "test_multi_fixture.py"), test_multi_fixture_content)
    create_file(os.path.join(project_name, "tests", "test_request_monkey_patch.py"), test_request_monkey_patch_content)
    create_file(os.path.join(project_name, "utils", "__init__.py"))
    create_file(os.path.join(project_name, "utils", "fastapi_mock.py"), fastapi_mock_content)
    create_file(os.path.join(project_name, "utils", "http_client.py"), http_client_content)
    create_file(os.path.join(project_name, "services", "__init__.py"))
    create_file(os.path.join(project_name, "services", "Login.py"), Login_content)
    create_file(os.path.join(project_name, "services", "SearchSku.py"), SearchSku_content)
    create_file(os.path.join(project_name, "services", "AddCart.py"), AddCart_content)
    create_file(os.path.join(project_name, "services", "Order.py"), Order_content)
    create_file(os.path.join(project_name, "services", "Pay.py"), Pay_content)

    if ExtraArgument.create_venv:
        os.chdir(project_name)
        print("\nCreating virtual environment")
        os.system("python -m venv .venv")
        print("Created virtual environment: .venv")

        print("Installing tep")
        if platform.system().lower() == 'windows':
            os.chdir(".venv")
            os.chdir("Scripts")
            os.system("pip install tep")
        elif platform.system().lower() == 'linux':
            os.chdir(".venv")
            os.chdir("bin")
            os.system("pip install tep")


def main_scaffold(args):
    ExtraArgument.create_venv = args.create_venv
    sys.exit(create_scaffold(args.project_name))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tep']

package_data = \
{'': ['*']}

install_requires = \
['allure-pytest>=2.8.16,<3.0.0',
 'allure-python-commons>=2.8.16,<3.0.0',
 'faker>=4.1.1,<5.0.0',
 'fastapi>=0.72.0,<0.73.0',
 'httprunner>=3.1.6,<4.0.0',
 'jmespath>=0.9.5,<0.10.0',
 'loguru>=0.4.1,<0.5.0',
 'pydantic>=1.9.0,<2.0.0',
 'pytest-assume>=2.4.2,<3.0.0',
 'pytest>=5.4.2,<6.0.0',
 'pyyaml>=5.4.1,<6.0.0',
 'requests>=2.22.0,<3.0.0',
 'urllib3>=1.25.9,<2.0.0',
 'uvicorn>=0.17.0,<0.18.0']

entry_points = \
{'console_scripts': ['tep = tep.cli:main'],
 'pytest11': ['tep = tep.plugin:Plugin']}

setup_kwargs = {
    'name': 'tep',
    'version': '0.9.6',
    'description': 'tep is a testing tool to help you write pytest more easily. Try Easy Pytest!',
    'long_description': '# tep\n\n`tep`是**T**ry **E**asy **P**ytest的首字母缩写，是一款基于pytest测试框架的测试工具，集成了各种实用的第三方包和优秀的自动化测试设计思想，帮你快速实现自动化项目落地。\n\n# 安装\n\n支持Python3.6以上，推荐Python3.8以上。\n\n标准安装：\n\n```\n$ pip install tep\n```\n\n国内镜像：\n\n```\n$ pip --default-timeout=600 install -i https://pypi.tuna.tsinghua.edu.cn/simple tep\n```\n\n检查安装成功：\n\n```\n$ tep -V  # 或者 tep --version\n0.2.3\n```\n\n# 快速创建项目\n\ntep提供了脚手架，预置了项目结构和代码，打开cmd，使用`startproject`命令快速创建项目：\n\n```\ntep startproject project_name\n```\n\n并且提供了`-venv`参数，在项目初始化时，可以同时创建一个虚拟环境（推荐）：\n\n```\ntep startproject project_name -venv\n```\n\n# 输出测试报告\n\ntep提供了`--tep-reports`参数来生成allure测试报告：\n\n```\npytest  --tep-reports\n```\n\n报告文件存放在根目录的`reports/`中。\n\n# Mock服务\n\ntep自带了一个Flask应用（`utils/flask_mock_api.py`），提供了登录到下单流程的5个接口，启动后即可一键运行示例中的测试用例。\n\n# 三种开发模式\n\ntep兼容三种开发模式：tep（用例数据一体）、mvc（用例数据分离）、HttpRunner。\n\n①tep，示例代码如下所示：\n\n```python\nimport jmespath\nfrom tep.client import request\n\n\ndef test(env_vars, login):\n    # 搜索商品\n    response = request(\n        "get",\n        url=env_vars.domain + "/searchSku",\n        headers={"token": login.token},\n        params={"skuName": "电子书"}\n    )\n    sku_id = jmespath.search("skuId", response.json())\n    sku_price = jmespath.search("price", response.json())\n    assert response.status_code < 400\n\n    # 添加购物车\n    sku_num = 3\n    response = request(\n        "post",\n        url=env_vars.domain + "/addCart",\n        headers={"token": login.token},\n        json={"skuId": sku_id, "skuNum": sku_num}\n    )\n    total_price = jmespath.search("totalPrice", response.json())\n    assert response.status_code < 400\n\n    # 下单\n    response = request(\n        "post",\n        url=env_vars.domain + "/order",\n        headers={"token": login.token},\n        json={"skuId": sku_id, "price": sku_price, "skuNum": sku_num, "totalPrice": total_price}\n    )\n    order_id = jmespath.search("orderId", response.json())\n    assert response.status_code < 400\n\n    # 支付\n    response = request(\n        "post",\n        url=env_vars.domain + "/pay",\n        headers={"token": login.token},\n        json={"orderId": order_id, "payAmount": "6.9"}\n    )\n    assert response.status_code < 400\n    assert response.json()["success"] == "true"\n\n```\n\n更多内容请参考[《如何使用teprunner测试平台编写从登录到下单的大流程接口自动化用例》](https://dongfanger.gitee.io/blog/teprunner/012-%E5%A6%82%E4%BD%95%E4%BD%BF%E7%94%A8teprunner%E6%B5%8B%E8%AF%95%E5%B9%B3%E5%8F%B0%E7%BC%96%E5%86%99%E4%BB%8E%E7%99%BB%E5%BD%95%E5%88%B0%E4%B8%8B%E5%8D%95%E7%9A%84%E5%A4%A7%E6%B5%81%E7%A8%8B%E6%8E%A5%E5%8F%A3%E8%87%AA%E5%8A%A8%E5%8C%96%E7%94%A8%E4%BE%8B.html)\n\n②mvc\n\n```python\nfrom tep.fixture import TepVars\n\nfrom services.AddCart import AddCart\nfrom services.Login import Login\nfrom services.Order import Order\nfrom services.Pay import Pay\nfrom services.SearchSku import SearchSku\n\n\\"\\"\\"\n测试登录到下单流程，需要先运行utils / flask_mock_api.py\n\\"\\"\\"\n\n\nclass Test:\n    case_vars = TepVars()\n    case_vars.vars_ = {\n        "domain": "http://127.0.0.1:5000",\n        "skuNum": "3"\n    }\n\n    def test(self):\n        # 登录\n        Login(Test).post()\n        # 搜索商品\n        SearchSku(Test).get()\n        # 添加购物车\n        AddCart(Test).post()\n        # 下单\n        Order(Test).post()\n        # 支付\n        Pay(Test).post()\n```\n\n③HttpRunner，示例代码如下所示：\n\n```python\nfrom httprunner import HttpRunner, Config, Step, RunRequest\n\n\nclass TestLoginPay(HttpRunner):\n    config = (\n        Config("登录到下单流程")\n            .variables(\n            **{\n                "skuNum": "3"\n            }\n        )\n            .base_url("http://127.0.0.1:5000")\n    )\n\n    teststeps = [\n        Step(\n            RunRequest("登录")\n                .post("/login")\n                .with_headers(**{"Content-Type": "application/json"})\n                .with_json({"username": "dongfanger", "password": "123456"})\n                .extract()\n                .with_jmespath("body.token", "token")\n                .validate()\n                .assert_equal("status_code", 200)\n        ),\n        Step(\n            RunRequest("搜索商品")\n                .get("searchSku?skuName=电子书")\n                .with_headers(**{"token": "$token"})\n                .extract()\n                .with_jmespath("body.skuId", "skuId")\n                .with_jmespath("body.price", "skuPrice")\n                .validate()\n                .assert_equal("status_code", 200)\n        ),\n        Step(\n            RunRequest("添加购物车")\n                .post("/addCart")\n                .with_headers(**{"Content-Type": "application/json",\n                                 "token": "$token"})\n                .with_json({"skuId": "$skuId", "skuNum": "$skuNum"})\n                .extract()\n                .with_jmespath("body.totalPrice", "totalPrice")\n                .validate()\n                .assert_equal("status_code", 200)\n        ),\n        Step(\n            RunRequest("下单")\n                .post("/order")\n                .with_headers(**{"Content-Type": "application/json",\n                                 "token": "$token"})\n                .with_json({"skuId": "$skuId", "price": "$skuPrice", "skuNum": "$skuNum", "totalPrice": "$totalPrice"})\n                .extract()\n                .with_jmespath("body.orderId", "orderId")\n                .validate()\n                .assert_equal("status_code", 200)\n        ),\n        Step(\n            RunRequest("支付")\n                .post("/pay")\n                .with_headers(**{"Content-Type": "application/json",\n                                 "token": "$token"})\n                .with_json({"orderId": "$orderId", "payAmount": "6.9"})\n                .validate()\n                .assert_equal("status_code", 200)\n                .assert_equal("body.success", "true")\n        ),\n    ]\n```\n\n# 猴子补丁扩展request\n\n扩展request，只需要实现`utils/http_client.py`里面的request_monkey_patch猴子补丁即可：\n\n```python\n#!/usr/bin/python\n# encoding=utf-8\n\nimport decimal\nimport json\nimport time\n\nimport allure\nfrom loguru import logger\nfrom tep import client\nfrom tep.client import TepResponse\n\n\ndef request_monkey_patch(req, *args, **kwargs):\n    start = time.process_time()\n    desc = ""\n    if "desc" in kwargs:\n        desc = kwargs.get("desc")\n        kwargs.pop("desc")\n    response = req(*args, **kwargs)\n    end = time.process_time()\n    elapsed = str(decimal.Decimal("%.3f" % float(end - start))) + "s"\n    log4a = "{}\\n{}status:{}\\nresponse:{}\\nelapsed:{}"\n    try:\n        kv = ""\n        for k, v in kwargs.items():\n            # if not json, str()\n            try:\n                v = json.dumps(v, ensure_ascii=False)\n            except TypeError:\n                v = str(v)\n            kv += f"{k}:{v}\\n"\n        if args:\n            method = f\'\\nmethod:"{args[0]}" \'\n        else:\n            method = ""\n        request_response = log4a.format(method, kv, response.status_code, response.text, elapsed)\n        logger.info(request_response)\n        allure.attach(request_response, f\'{desc} request & response\', allure.attachment_type.TEXT)\n    except AttributeError:\n        logger.error("request failed")\n    except TypeError:\n        logger.warning(log4a)\n    return TepResponse(response)\n\n\ndef request(method, url, **kwargs):\n    client.tep_request_monkey_patch = request_monkey_patch\n    return client.request(method, url, **kwargs)\n```\n\n# 可选第三方包安装\n\n```\n# pip install --default-timeout=6000 -i https://pypi.tuna.tsinghua.edu.cn/simple pandas\n\n# mysql\npandas==1.1.0\nSQLAlchemy==1.3.19\nPyMySQL==0.10.0\ntexttable==1.6.2\n\n# more\n```\n\n# 用户手册\n\nhttps://dongfanger.gitee.io/blog/chapters/tep.html\n\n# 联系我\n\nhttps://dongfanger.gitee.io/blog/more.html\n\n',
    'author': 'dongfanger',
    'author_email': 'dongfanger@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dongfanger/tep',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
